@@ -27,27 +27,27 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <config.h>
 
-#include <errno.h>
 #include <fcntl.h>
 #include <stdbool.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/select.h>
+#ifndef WIN32
 #include <unistd.h>
+#else
+#include <process.h>
+#define snprintf sprintf_s
+#endif
 
-#include <mqtt3.h>
-#include <client_shared.h>
+#include <mosquitto.h>
 
 #define MSGMODE_NONE 0
 #define MSGMODE_CMD 1
 #define MSGMODE_STDIN_LINE 2
 #define MSGMODE_STDIN_FILE 3
 #define MSGMODE_FILE 4
+#define MSGMODE_NULL 5
 
 #define STATUS_CONNECTING 0
 #define STATUS_CONNACK_RECVD 1
@@ -57,18 +57,24 @@ static char *message = NULL;
 static long msglen = 0;
 static int qos = 0;
 static int retain = 0;
-static mqtt3_context *gcontext;
 static int mode = MSGMODE_NONE;
 static int status = STATUS_CONNECTING;
+static uint16_t mid_sent = 0;
+static bool connected = true;
 
-void my_connack_callback(int result)
+void my_connect_callback(void *obj, int result)
 {
+	struct mosquitto *mosq = obj;
+
 	if(!result){
 		switch(mode){
 			case MSGMODE_CMD:
 			case MSGMODE_FILE:
 			case MSGMODE_STDIN_FILE:
-				mqtt3_raw_publish(gcontext, false, qos, retain, 1, topic, msglen, (uint8_t *)message);
+				mosquitto_publish(mosq, &mid_sent, topic, msglen, (uint8_t *)message, qos, retain);
+				break;
+			case MSGMODE_NULL:
+				mosquitto_publish(mosq, &mid_sent, topic, 0, NULL, qos, retain);
 				break;
 			case MSGMODE_STDIN_LINE:
 				status = STATUS_CONNACK_RECVD;
@@ -79,24 +85,17 @@ void my_connack_callback(int result)
 	}
 }
 
-void my_net_write_callback(int command)
+void my_disconnect_callback(void *obj)
 {
-	if(qos == 0 && command == PUBLISH && mode != MSGMODE_STDIN_LINE){
-		mqtt3_raw_disconnect(gcontext);
-	}
+	connected = false;
 }
 
-void my_puback_callback(int mid)
+void my_publish_callback(void *obj, uint16_t mid)
 {
-	if(mode != MSGMODE_STDIN_LINE){
-		mqtt3_raw_disconnect(gcontext);
-	}
-}
+	struct mosquitto *mosq = obj;
 
-void my_pubcomp_callback(int mid)
-{
 	if(mode != MSGMODE_STDIN_LINE){
-		mqtt3_raw_disconnect(gcontext);
+		mosquitto_disconnect(mosq);
 	}
 }
 
@@ -109,7 +108,7 @@ int load_stdin(void)
 
 	while(!feof(stdin)){
 		rlen = fread(buf, 1, 1024, stdin);
-		message = mqtt3_realloc(message, pos+rlen);
+		message = realloc(message, pos+rlen);
 		if(!message){
 			fprintf(stderr, "Error: Out of memory.\n");
 			return 1;
@@ -151,7 +150,7 @@ int load_file(const char *filename)
 		return 1;
 	}
 	fseek(fptr, 0, SEEK_SET);
-	message = mqtt3_malloc(msglen);
+	message = malloc(msglen);
 	if(!message){
 		fclose(fptr);
 		fprintf(stderr, "Error: Out of memory.\n");
@@ -169,22 +168,30 @@ int load_file(const char *filename)
 void print_usage(void)
 {
 	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n\n");
-	printf("Usage: mosquitto_pub [-h host] [-i id] [-p port] [-q qos] [-r] {-f file | -l | -m message} -t topic\n\n");
+	printf("Usage: mosquitto_pub [-d] [-h host] [-i id] [-p port] [-q qos] [-r] {-f file | -l | -n | -m message} -t topic\n");
+	printf("                     [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n\n");
+	printf(" -d : enable debug messages.\n");
 	printf(" -f : send the contents of a file as the message.\n");
 	printf(" -h : mqtt host to connect to. Defaults to localhost.\n");
 	printf(" -i : id to use for this client. Defaults to mosquitto_pub_ appended with the process id.\n");
 	printf(" -l : read messages from stdin, sending a separate message for each line.\n");
 	printf(" -m : message payload to send.\n");
+	printf(" -n : send a null (zero length) message.\n");
 	printf(" -p : network port to connect to. Defaults to 1883.\n");
 	printf(" -q : quality of service level to use for all messages. Defaults to 0.\n");
 	printf(" -r : message should be retained.\n");
 	printf(" -s : read message from stdin, sending the entire input as a message.\n");
 	printf(" -t : mqtt topic to publish to.\n");
+	printf(" --will-payload : payload for the client Will, which is sent by the broker in case of\n");
+	printf("                  unexpected disconnection. If not given and will-topic is set, a zero\n");
+	printf("                  length message will be sent.\n");
+	printf(" --will-qos : QoS level for the client Will.\n");
+	printf(" --will-retain : if given, make the client Will retained.\n");
+	printf(" --will-topic : the topic on which to publish the client Will.\n");
 }
 
 int main(int argc, char *argv[])
 {
-	mqtt3_context *context;
 	char id[30];
 	int i;
 	char *host = "localhost";
@@ -192,9 +199,16 @@ int main(int argc, char *argv[])
 	int keepalive = 60;
 	int opt;
 	char buf[1024];
-	int mid_sent = 0;
+	bool debug = false;
+	struct mosquitto *mosq = NULL;
 
-	sprintf(id, "mosquitto_pub_%d", getpid());
+	uint8_t *will_payload = NULL;
+	long will_payloadlen = 0;
+	int will_qos = 0;
+	bool will_retain = false;
+	char *will_topic = NULL;
+
+	snprintf(id, 30, "mosquitto_pub_%d", getpid());
 
 	for(i=1; i<argc; i++){
 		if(!strcmp(argv[i], "-p") || !strcmp(argv[i], "--port")){
@@ -211,6 +225,8 @@ int main(int argc, char *argv[])
 				}
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-d") || !strcmp(argv[i], "--debug")){
+			debug = true;
 		}else if(!strcmp(argv[i], "-f") || !strcmp(argv[i], "--file")){
 			if(mode != MSGMODE_NONE){
 				fprintf(stderr, "Error: Only one type of message can be sent at once.\n\n");
@@ -250,11 +266,13 @@ int main(int argc, char *argv[])
 				return 1;
 			}else{
 				mode = MSGMODE_STDIN_LINE;
+#ifndef WIN32
 				opt = fcntl(fileno(stdin), F_GETFL, 0);
 				if(opt == -1 || fcntl(fileno(stdin), F_SETFL, opt | O_NONBLOCK) == -1){
 					fprintf(stderr, "Error: Unable to set stdin to non-blocking.\n");
 					return 1;
 				}
+#endif
 			}
 		}else if(!strcmp(argv[i], "-m") || !strcmp(argv[i], "--message")){
 			if(mode != MSGMODE_NONE){
@@ -271,6 +289,14 @@ int main(int argc, char *argv[])
 				mode = MSGMODE_CMD;
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-n") || !strcmp(argv[i], "--null-message")){
+			if(mode != MSGMODE_NONE){
+				fprintf(stderr, "Error: Only one type of message can be sent at once.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				mode = MSGMODE_NULL;
+			}
 		}else if(!strcmp(argv[i], "-q") || !strcmp(argv[i], "--qos")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: -q argument given but no QoS specified.\n\n");
@@ -304,51 +330,100 @@ int main(int argc, char *argv[])
 				topic = argv[i+1];
 			}
 			i++;
+		}else if(!strcmp(argv[i], "--will-payload")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --will-payload argument given but no will payload specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				will_payload = (uint8_t *)argv[i+1];
+				will_payloadlen = strlen((char *)will_payload);
+			}
+			i++;
+		}else if(!strcmp(argv[i], "--will-qos")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --will-qos argument given but no will QoS specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				will_qos = atoi(argv[i+1]);
+				if(will_qos < 0 || will_qos > 2){
+					fprintf(stderr, "Error: Invalid will QoS %d.\n\n", will_qos);
+					return 1;
+				}
+			}
+			i++;
+		}else if(!strcmp(argv[i], "--will-retain")){
+			will_retain = true;
+		}else if(!strcmp(argv[i], "--will-topic")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --will-topic argument given but no will topic specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				will_topic = argv[i+1];
+			}
+			i++;
 		}else{
 			fprintf(stderr, "Error: Unknown option '%s'.\n",argv[i]);
 			print_usage();
 			return 1;
 		}
 	}
-	fflush(stdout);
 	if(!topic || mode == MSGMODE_NONE){
 		fprintf(stderr, "Error: Both topic and message must be supplied.\n");
 		print_usage();
 		return 1;
 	}
 
-	if(client_init()){
-		fprintf(stderr, "Error: Unable to initialise database.\n");
+	if(will_payload && !will_topic){
+		fprintf(stderr, "Error: Will payload given, but no will topic given.\n");
+		print_usage();
+		return 1;
+	}
+	if(will_retain && !will_topic){
+		fprintf(stderr, "Error: Will retain given, but no will topic given.\n");
+		print_usage();
+		return 1;
+	}
+	mosquitto_lib_init();
+	mosq = mosquitto_new(id, NULL);
+	if(!mosq){
+		fprintf(stderr, "Error: Out of memory.\n");
+		return 1;
+	}
+	if(debug){
+		mosquitto_log_init(mosq, MOSQ_LOG_DEBUG | MOSQ_LOG_ERR | MOSQ_LOG_WARNING
+				| MOSQ_LOG_NOTICE | MOSQ_LOG_INFO, MOSQ_LOG_STDERR);
+	}
+	if(will_topic && mosquitto_will_set(mosq, true, will_topic, will_payloadlen, will_payload, will_qos, will_retain)){
+		fprintf(stderr, "Error: Problem setting will.\n");
 		return 1;
 	}
-	client_connack_callback = my_connack_callback;
-	client_net_write_callback = my_net_write_callback;
-	client_puback_callback = my_puback_callback;
-	client_pubcomp_callback = my_pubcomp_callback;
 
-	if(client_connect(&context, host, port, id, keepalive, true)){
+	mosquitto_connect_callback_set(mosq, my_connect_callback);
+	mosquitto_disconnect_callback_set(mosq, my_disconnect_callback);
+	mosquitto_publish_callback_set(mosq, my_publish_callback);
+
+	if(mosquitto_connect(mosq, host, port, keepalive, true)){
 		fprintf(stderr, "Unable to connect.\n");
 		return 1;
 	}
-	gcontext = context;
 
-	while(!client_loop(context)){
+	while(!mosquitto_loop(mosq, -1) && connected){
 		if(mode == MSGMODE_STDIN_LINE && status == STATUS_CONNACK_RECVD){
 			if(fgets(buf, 1024, stdin)){
 				buf[strlen(buf)-1] = '\0';
-				if(strlen(buf) > 0){
-					mid_sent++;
-					if(mid_sent > 65535) mid_sent = 1;
-					mqtt3_raw_publish(context, false, qos, retain, mid_sent, topic, strlen(buf), (uint8_t *)buf);
-				}
+				mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), (uint8_t *)buf, qos, retain);
 			}else if(feof(stdin)){
-				mqtt3_raw_disconnect(gcontext);
+				mosquitto_disconnect(mosq);
 			}
 		}
 	}
 	if(message && mode == MSGMODE_FILE){
-		mqtt3_free(message);
+		free(message);
 	}
-	client_cleanup();
+	mosquitto_destroy(mosq);
+	mosquitto_lib_cleanup();
 	return 0;
 }
