@@ -74,6 +74,7 @@ int _mosquitto_message_delete(struct mosquitto *mosq, uint16_t mid, enum mosquit
 				mosq->messages = message->next;
 			}
 			_mosquitto_message_cleanup(&message);
+			return 0;
 		}
 		prev = message;
 		message = message->next;
@@ -130,7 +131,7 @@ void _mosquitto_message_retry_check(struct mosquitto *mosq)
 
 	message = mosq->messages;
 	while(message){
-		if(message->timestamp + mosq->message_retry > now){
+		if(message->timestamp + mosq->message_retry < now){
 			switch(message->state){
 				case mosq_ms_wait_puback:
 				case mosq_ms_wait_pubrec:
@@ -171,6 +172,7 @@ int _mosquitto_message_update(struct mosquitto *mosq, uint16_t mid, enum mosquit
 			message->timestamp = time(NULL);
 			return 0;
 		}
+		message = message->next;
 	}
 	return 1;
 }
