@@ -30,26 +30,29 @@
 import mosquitto
 from ctypes import *
 
-def py_on_connect(obj, rc):
+def on_connect(rc):
 	print "rc: ", rc
 
+def on_message(msg):
+	print msg.topic,msg.qos,msg.payload
 
-def py_on_message(obj, message):
-	print message.contents.topic,message.contents.payload
-
-def py_on_publish(obj, mid):
+def on_publish(mid):
 	print "mid:", mid
 
+def on_subscribe(mid, granted_qos):
+	print "Subscribed:",mid,granted_qos
+	
 
 mqttc = mosquitto.Mosquitto("python_sub")
-mqttc.connect_callback(py_on_connect)
-mqttc.message_callback(py_on_message)
-mqttc.publish_callback(py_on_publish)
+mqttc.on_message = on_message
+mqttc.on_connect = on_connect
+mqttc.on_publish = on_publish
+mqttc.on_subscribe = on_subscribe
 mqttc.connect("127.0.0.1", 1883, 60, True)
 mqttc.subscribe("$SYS/#", 2)
 mqttc.subscribe("#", 2)
-mqttc.publish("bob", 8, cast("0000000", POINTER(c_uint8)))
+mqttc.publish("bob", "0000000")
 
-while 1:
-	mqttc.loop()
+while mqttc.loop() == 0:
+	pass
 
