@@ -22,6 +22,7 @@ void mqtt3_config_init(mqtt3_config *config)
 	config->iface_count = 0;
 	config->log_dest = MQTT3_LOG_STDERR;
 	config->log_type = MQTT3_LOG_ERR | MQTT3_LOG_WARNING | MQTT3_LOG_NOTICE | MQTT3_LOG_INFO;
+	config->max_connections = -1;
 	config->persistence = false;
 	config->persistence_location = NULL;
 	config->persistence_file = "mosquitto.db";
@@ -140,7 +141,7 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 			}
 			token = strtok(buf, " ");
 			if(token){
-				if(!strcmp(token, "address")){
+				if(!strcmp(token, "address") || !strcmp(token, "addresses")){
 					if(!cur_bridge || cur_bridge->address){
 						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid bridge configuration.");
 						return 1;
@@ -181,6 +182,12 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty ext_sqlite_regex value in config.");
 						return 1;
 					}
+				}else if(!strcmp(token, "cleansession")){
+					if(!cur_bridge){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid bridge configuration.");
+						return 1;
+					}
+					if(_mqtt3_conf_parse_bool(&token, "cleansession", &cur_bridge->clean_session)) return 1;
 				}else if(!strcmp(token, "connection")){
 					token = strtok(NULL, " ");
 					if(token){
@@ -193,6 +200,8 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 						cur_bridge = &(config->bridges[config->bridge_count-1]);
 						cur_bridge->name = mqtt3_strdup(token);
 						cur_bridge->address = NULL;
+						cur_bridge->keepalive = 60;
+						cur_bridge->clean_session = false;
 						cur_bridge->port = 0;
 						cur_bridge->topics = NULL;
 						cur_bridge->topic_count = 0;
@@ -229,6 +238,16 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty interface value in configuration.");
 						return 1;
 					}
+				}else if(!strcmp(token, "keepalive_interval")){
+					if(!cur_bridge){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid bridge configuration.");
+						return 1;
+					}
+					if(_mqtt3_conf_parse_int(&token, "keepalive_interval", &cur_bridge->keepalive)) return 1;
+					if(cur_bridge->keepalive < 5){
+						mqtt3_log_printf(MQTT3_LOG_NOTICE, "keepalive interval too low, using 5 seconds.");
+						cur_bridge->keepalive = 5;
+					}
 				}else if(!strcmp(token, "log_dest")){
 					token = strtok(NULL, " ");
 					if(token){
@@ -274,6 +293,14 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 					}else{
 						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty log_type value in configuration.");
 					}
+				}else if(!strcmp(token, "max_connections")){
+					token = strtok(NULL, " ");
+					if(token){
+						config->max_connections = atoi(token);
+						if(config->max_connections < 0) config->max_connections = -1;
+					}else{
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty max_connections value in configuration.");
+					}
 				}else if(!strcmp(token, "max_inflight_messages")){
 					token = strtok(NULL, " ");
 					if(token){
@@ -395,13 +422,10 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 						|| !strcmp(token, "clientid_prefixes")
 						|| !strcmp(token, "connection_messages")
 						|| !strcmp(token, "listener")
-						|| !strcmp(token, "max_connections")
 						|| !strcmp(token, "retained_persistence")
 						|| !strcmp(token, "trace_level")
 						|| !strcmp(token, "addresses")
-						|| !strcmp(token, "cleansession")
 						|| !strcmp(token, "idle_timeout")
-						|| !strcmp(token, "keepalive_interval")
 						|| !strcmp(token, "notifications")
 						|| !strcmp(token, "notification_topic")
 						|| !strcmp(token, "round_robin")
@@ -420,6 +444,7 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 	fclose(fptr);
 
 	mqtt3_db_limits_set(max_inflight_messages, max_queued_messages);
+	mqtt3_net_set_max_connections(config->max_connections);
 
 	for(i=0; i<config->bridge_count; i++){
 		if(!config->bridges[i].name || !config->bridges[i].address || !config->bridges[i].port || !config->bridges[i].topic_count){
