@@ -31,11 +31,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #define MQTT3_H
 
 #include <config.h>
+#include <net_mosq.h>
 
 #include <stdbool.h>
 #include <stdint.h>
 #include <syslog.h>
-#include <sys/select.h>
 #include <time.h>
 
 #ifndef __GNUC__
@@ -50,10 +50,6 @@ POSSIBILITY OF SUCH DAMAGE.
 /* Database macros */
 #define MQTT_DB_VERSION 2
 
-/* Macros for accessing the MSB and LSB of a uint16_t */
-#define MQTT_MSB(A) (uint8_t)((A & 0xFF00) >> 8)
-#define MQTT_LSB(A) (uint8_t)(A & 0x00FF)
-
 /* Message types */
 #define CONNECT 0x10
 #define CONNACK 0x20
@@ -86,36 +82,23 @@ POSSIBILITY OF SUCH DAMAGE.
 #define MQTT3_LOG_ERR 0x08
 #define MQTT3_LOG_DEBUG 0x10
 
-struct _mqtt3_packet{
-	uint8_t command;
-#ifdef WITH_CLIENT
-	uint8_t command_saved;
-#endif
-	uint8_t have_remaining;
-	uint8_t remaining_count;
-	uint32_t remaining_mult;
-	uint32_t remaining_length;
-	uint32_t to_process;
-	uint32_t pos;
-	uint8_t *payload;
-	struct _mqtt3_packet *next;
-};
-
-typedef enum {
+enum mqtt3_bridge_direction{
 	bd_out = 0,
 	bd_in = 1,
 	bd_both = 2
-} mqtt3_bridge_direction;
+};
 
 struct _mqtt3_bridge_topic{
 	char *topic;
-	mqtt3_bridge_direction direction;
+	enum mqtt3_bridge_direction direction;
 };
 
 struct _mqtt3_bridge{
 	char *name;
 	char *address;
 	uint16_t port;
+	int keepalive;
+	bool clean_session;
 	struct _mqtt3_bridge_topic *topics;
 	int topic_count;
 	time_t restart_t;
@@ -129,14 +112,15 @@ typedef struct _mqtt3_context{
 	bool clean_session;
 	bool connected;
 	bool disconnecting;
+	bool duplicate;
 	char *id;
 	char *address;
-	struct _mqtt3_packet in_packet;
-	struct _mqtt3_packet *out_packet;
+	struct _mosquitto_packet in_packet;
+	struct _mosquitto_packet *out_packet;
 	struct _mqtt3_bridge *bridge;
 } mqtt3_context;
 
-typedef enum {
+enum mqtt3_msg_status {
 	ms_invalid = 0,
 	ms_publish = 1,
 	ms_publish_puback = 2,
@@ -147,12 +131,7 @@ typedef enum {
 	ms_wait_pubrel = 7,
 	ms_resend_pubcomp = 8,
 	ms_wait_pubcomp = 9
-} mqtt3_msg_status;
-
-typedef enum {
-	md_in = 0,
-	md_out = 1
-} mqtt3_msg_direction;
+};
 
 struct mqtt3_iface {
 	char *iface;
@@ -167,6 +146,7 @@ typedef struct {
 	int iface_count;
 	int log_dest;
 	int log_type;
+	int max_connections;
 	bool persistence;
 	char *persistence_location;
 	char *persistence_file;
@@ -188,27 +168,12 @@ struct _mqtt3_listener {
 	char *client_prefix;
 };
 
-#ifdef WITH_CLIENT
-/* Client callback for publish events - this WILL change. */
-extern int (*client_publish_callback)(const char *, int, uint32_t, const uint8_t *, int);
-/* Client callback for connack events - this WILL change. */
-extern void (*client_connack_callback)(int);
-/* Client callback for puback events - this WILL change. */
-extern void (*client_puback_callback)(int);
-/* Client callback for pubcomp events - this WILL change. */
-extern void (*client_pubcomp_callback)(int);
-/* Client callback for when a packet has been successfully written. */
-extern void (*client_net_write_callback)(int);
-#endif
-
 /* ============================================================
  * Utility functions
  * ============================================================ */
 /* Return a string that corresponds to the MQTT command number (left shifted 4 bits). */
 const char *mqtt3_command_to_string(uint8_t command);
 void mqtt3_check_keepalive(mqtt3_context *context);
-/* Remove excessive slashes in a subscription/topic. */
-int mqtt3_fix_sub_topic(char **subtopic);
 
 /* ============================================================
  * Config functions
@@ -247,25 +212,16 @@ int mqtt3_send_simple_command(mqtt3_context *context, uint8_t command);
 /* ============================================================
  * Network functions
  * ============================================================ */
-int mqtt3_socket_accept(mqtt3_context **contexts, int *context_count, int listensock);
-int mqtt3_socket_connect(const char *host, uint16_t port);
+int mqtt3_socket_accept(mqtt3_context ***contexts, int *context_count, int listensock);
 int mqtt3_socket_close(mqtt3_context *context);
 int mqtt3_socket_listen(uint16_t port);
 int mqtt3_socket_listen_if(const char *iface, uint16_t port);
 
-int mqtt3_net_packet_queue(mqtt3_context *context, struct _mqtt3_packet *packet);
+int mqtt3_net_packet_queue(mqtt3_context *context, struct _mosquitto_packet *packet);
 int mqtt3_net_read(mqtt3_context *context);
 int mqtt3_net_write(mqtt3_context *context);
-int mqtt3_read_byte(mqtt3_context *context, uint8_t *byte);
-int mqtt3_read_bytes(mqtt3_context *context, uint8_t *bytes, uint32_t count);
-int mqtt3_read_string(mqtt3_context *context, char **str);
-int mqtt3_read_uint16(mqtt3_context *context, uint16_t *word);
-
-int mqtt3_write_byte(struct _mqtt3_packet *packet, uint8_t byte);
-int mqtt3_write_bytes(struct _mqtt3_packet *packet, const uint8_t *bytes, uint32_t count);
-int mqtt3_write_string(struct _mqtt3_packet *packet, const char *str, uint16_t length);
-int mqtt3_write_uint16(struct _mqtt3_packet *packet, uint16_t word);
 
+void mqtt3_net_set_max_connections(int max);
 uint64_t mqtt3_net_bytes_total_received(void);
 uint64_t mqtt3_net_bytes_total_sent(void);
 unsigned long mqtt3_net_msgs_total_received(void);
@@ -310,11 +266,11 @@ int mqtt3_db_client_will_queue(mqtt3_context *context);
 void mqtt3_db_limits_set(int inflight, int queued);
 /* Return the number of in-flight messages in count. */
 int mqtt3_db_message_count(int *count);
-int mqtt3_db_message_delete(const char *client_id, uint16_t mid, mqtt3_msg_direction dir);
+int mqtt3_db_message_delete(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir);
 int mqtt3_db_message_delete_by_oid(int64_t oid);
-int mqtt3_db_message_insert(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status, int qos, int64_t store_id);
-int mqtt3_db_message_release(const char *client_id, uint16_t mid, mqtt3_msg_direction dir);
-int mqtt3_db_message_update(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status);
+int mqtt3_db_message_insert(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir, enum mqtt3_msg_status status, int qos, int64_t store_id);
+int mqtt3_db_message_release(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir);
+int mqtt3_db_message_update(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir, enum mqtt3_msg_status status);
 int mqtt3_db_message_write(mqtt3_context *context);
 int mqtt3_db_messages_delete(const char *client_id);
 int mqtt3_db_messages_easy_queue(const char *client_id, const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain);
@@ -333,18 +289,18 @@ int mqtt3_db_store_clean(void);
 int mqtt3_db_sub_insert(const char *client_id, const char *sub, int qos);
 /* Remove a subscription for a client. */
 int mqtt3_db_sub_delete(const char *client_id, const char *sub);
-int mqtt3_db_sub_search_start(const char *source_id, const char *topic);
-int mqtt3_db_sub_search_next(char **client_id, uint8_t *qos);
+int mqtt3_db_sub_search_start(const char *source_id, const char *topic, int qos);
 /* Remove all subscriptions for a client. */
 int mqtt3_db_subs_clean_session(const char *client_id);
 void mqtt3_db_sys_update(int interval, time_t start_time);
+void mqtt3_db_vacuum(void);
 
 /* ============================================================
  * Context functions
  * ============================================================ */
 mqtt3_context *mqtt3_context_init(int sock);
 void mqtt3_context_cleanup(mqtt3_context *context);
-void mqtt3_context_packet_cleanup(struct _mqtt3_packet *packet);
+void mqtt3_context_close_duplicate(int sock);
 
 /* ============================================================
  * Memory functions
