@@ -27,16 +27,6 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <errno.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/select.h>
-#include <unistd.h>
-
 #include <config.h>
 #include <mqtt3.h>
 
@@ -48,9 +38,12 @@ int mqtt3_handle_connack(mqtt3_context *context)
 	uint8_t rc;
 	int i;
 
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received CONNACK");
-	if(mqtt3_read_byte(context, &byte)) return 1; // Reserved byte, not used
-	if(mqtt3_read_byte(context, &rc)) return 1;
+	if(_mosquitto_read_byte(&context->in_packet, &byte)) return 1; // Reserved byte, not used
+	if(_mosquitto_read_byte(&context->in_packet, &rc)) return 1;
 	if(client_connack_callback){
 		client_connack_callback(rc);
 	}
@@ -65,6 +58,7 @@ int mqtt3_handle_connack(mqtt3_context *context)
 					}
 				}
 			}
+			context->connected = true;
 			return 0;
 		case 1:
 			mqtt3_log_printf(MQTT3_LOG_ERR, "Connection Refused: unacceptable protocol version");
@@ -85,11 +79,11 @@ int mqtt3_handle_suback(mqtt3_context *context)
 	uint8_t granted_qos;
 
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received SUBACK");
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 
 	while(context->in_packet.pos < context->in_packet.remaining_length){
 		/* FIXME - Need to do something with this */
-		if(mqtt3_read_byte(context, &granted_qos)) return 1;
+		if(_mosquitto_read_byte(&context->in_packet, &granted_qos)) return 1;
 	}
 
 	return 0;
@@ -99,8 +93,11 @@ int mqtt3_handle_unsuback(mqtt3_context *context)
 {
 	uint16_t mid;
 
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received UNSUBACK");
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 
 	return 0;
 }
