@@ -27,18 +27,11 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <errno.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/select.h>
-#include <unistd.h>
 
 #include <config.h>
 #include <mqtt3.h>
+#include <util_mosq.h>
 
 int mqtt3_handle_connect(mqtt3_context *context)
 {
@@ -52,7 +45,7 @@ int mqtt3_handle_connect(mqtt3_context *context)
 	/* Don't accept multiple CONNECT commands. */
 	if(context->connected) return 1;
 
-	if(mqtt3_read_string(context, &protocol_name)) return 1;
+	if(_mosquitto_read_string(&context->in_packet, &protocol_name)) return 1;
 	if(!protocol_name){
 		mqtt3_socket_close(context);
 		return 3;
@@ -64,7 +57,7 @@ int mqtt3_handle_connect(mqtt3_context *context)
 		mqtt3_socket_close(context);
 		return 1;
 	}
-	if(mqtt3_read_byte(context, &protocol_version)) return 1;
+	if(_mosquitto_read_byte(&context->in_packet, &protocol_version)) return 1;
 	if(protocol_version != PROTOCOL_VERSION){
 		mqtt3_log_printf(MQTT3_LOG_INFO, "Invalid protocol version %d in CONNECT from %s.",
 				protocol_version, context->address);
@@ -76,18 +69,18 @@ int mqtt3_handle_connect(mqtt3_context *context)
 
 	mqtt3_free(protocol_name);
 
-	if(mqtt3_read_byte(context, &connect_flags)) return 1;
+	if(_mosquitto_read_byte(&context->in_packet, &connect_flags)) return 1;
 	clean_session = connect_flags & 0x02;
 	will = connect_flags & 0x04;
-	will_qos = (connect_flags & 0x18) >> 2;
+	will_qos = (connect_flags & 0x18) >> 3;
 	will_retain = connect_flags & 0x20;
 
-	if(mqtt3_read_uint16(context, &(context->keepalive))) return 1;
+	if(_mosquitto_read_uint16(&context->in_packet, &(context->keepalive))) return 1;
 
-	if(mqtt3_read_string(context, &client_id)) return 1;
+	if(_mosquitto_read_string(&context->in_packet, &client_id)) return 1;
 	if(connect_flags & 0x04){
-		if(mqtt3_read_string(context, &will_topic)) return 1;
-		if(mqtt3_read_string(context, &will_message)) return 1;
+		if(_mosquitto_read_string(&context->in_packet, &will_topic)) return 1;
+		if(_mosquitto_read_string(&context->in_packet, &will_message)) return 1;
 	}
 
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received CONNECT from %s as %s", context->address, client_id);
@@ -105,6 +98,9 @@ int mqtt3_handle_connect(mqtt3_context *context)
 
 int mqtt3_handle_disconnect(mqtt3_context *context)
 {
+	if(!context || context->in_packet.remaining_length != 0){
+		return 1;
+	}
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received DISCONNECT from %s", context->id);
 	context->disconnecting = true;
 	return mqtt3_socket_close(context);
@@ -120,27 +116,27 @@ int mqtt3_handle_subscribe(mqtt3_context *context)
 	uint8_t *payload = NULL;
 	uint32_t payloadlen = 0;
 
+	if(!context) return 1;
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received SUBSCRIBE from %s", context->id);
 	/* FIXME - plenty of potential for memory leaks here */
-	if(!context) return 1;
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 
 	while(context->in_packet.pos < context->in_packet.remaining_length){
 		sub = NULL;
-		if(mqtt3_read_string(context, &sub)){
+		if(_mosquitto_read_string(&context->in_packet, &sub)){
 			if(sub) mqtt3_free(sub);
 			if(payload) mqtt3_free(payload);
 			return 1;
 		}
 
 		if(sub){
-			if(mqtt3_read_byte(context, &qos)){
+			if(_mosquitto_read_byte(&context->in_packet, &qos)){
 				mqtt3_free(sub);
 				if(payload) mqtt3_free(payload);
 				return 1;
 			}
-			if(mqtt3_fix_sub_topic(&sub)){
+			if(_mosquitto_fix_sub_topic(&sub)){
 				mqtt3_free(sub);
 				if(payload) mqtt3_free(payload);
 				return 1;
@@ -175,14 +171,14 @@ int mqtt3_handle_unsubscribe(mqtt3_context *context)
 	uint16_t mid;
 	char *sub;
 
-	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received UNSUBSCRIBE from %s", context->id);
 	if(!context) return 1;
+	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received UNSUBSCRIBE from %s", context->id);
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 
 	while(context->in_packet.pos < context->in_packet.remaining_length){
 		sub = NULL;
-		if(mqtt3_read_string(context, &sub)){
+		if(_mosquitto_read_string(&context->in_packet, &sub)){
 			if(sub) mqtt3_free(sub);
 			return 1;
 		}
