@@ -71,36 +71,12 @@ const char *mqtt3_command_to_string(uint8_t command)
 
 void mqtt3_check_keepalive(mqtt3_context *context)
 {
-	if(time(NULL) - context->last_msg_out >= context->keepalive){
-		mqtt3_raw_pingreq(context);
+	if(context && context->sock != -1 && time(NULL) - context->last_msg_out >= context->keepalive){
+		if(context->connected){
+			mqtt3_raw_pingreq(context);
+		}else{
+			mqtt3_socket_close(context);
+		}
 	}
 }
 
-/* Convert ////some////over/slashed///topic/etc/etc//
- * into some/over/slashed/topic/etc/etc
- */
-int mqtt3_fix_sub_topic(char **subtopic)
-{
-	char *fixed = NULL;
-	char *token;
-
-	if(!subtopic || !(*subtopic)) return 1;
-
-	/* size of fixed here is +1 for the terminating 0 and +1 for the spurious /
-	 * that gets appended. */
-	fixed = mqtt3_calloc(strlen(*subtopic)+2, 1);
-	if(!fixed) return 1;
-
-	token = strtok(*subtopic, "/");
-	while(token){
-		strcat(fixed, token);
-		strcat(fixed, "/");
-		token = strtok(NULL, "/");
-	}
-
-	fixed[strlen(fixed)-1] = '\0';
-	mqtt3_free(*subtopic);
-	*subtopic = fixed;
-	return 0;
-}
-
